/**************************************************************************
	CxTranslator V4.
***************************************************************************/
#pragma once

#include <TCHAR.H>
#include <windows.h>

#if !defined _USE_CXTRANSLATOR 
#define _USE_CXTRANSLATOR
#endif

#include "EquTypes.h"



//Handle to an EquTranslator object
typedef void* CXHANDLE;


#if defined(__cplusplus)
 extern "C"
 {
#endif


/**************************************************************************
      Initialize EquTranslator algorithm.
        This call sets up an init state for the algorithm and allocates memory. 
        When EquTranslator is not required, CloseCxTranslator() must be called.
        Arguments:
              errFunction  callback function for error management. 
        Return:
               Handle to context.
***************************************************************************/
CXHANDLE  __stdcall InitCxTranslator
(
	TpfErrorHandler errFunction				//Error callback function
);



/**************************************************************************
	Clear resources associated with EquTranslator for a given context. 
	 Arguments:
		equHandle - handle to context. 
***************************************************************************/
void __stdcall CloseCxTranslator(CXHANDLE CXHANDLE);


/**************************************************************************
	    Parse and evaluate a given math expression.
        Arguments:
			equHandle - handle to context. 
            mathExpression  math expression as string to be evaluated.
            varList  coma separated list of named variables. For example x,y,z.
            arguments  array of values for a given varList. First element in the array 
                      corresponds to first named variable in varList, second to second...
        Return:
               Result of evaluation.
***************************************************************************/
TCalc __stdcall Evaluate(CXHANDLE CXHANDLE, const TCHAR *mathExpression, const TCHAR* varList, const TCalc *arguments);



/**************************************************************************
      Parse math expression.
        The function prepares math expression but doesnt evaluate it. 
        To evaluate one for a given values CalcFor() should be called. 
        Functions Build() and CalcFor() should be used when the same math 
        expression must be evaluated for a large set of points. If number of 
        points is small (not more than 10)  function Evaluate() may work faster.
        Arguments:
			equHandle - handle to context.
            mathExpression  math expression as string to be parsed.
            varList  coma separated list of named variables. For example x,y,z.
        Return:
               True if call succeeded.
***************************************************************************/
bool __stdcall Build(CXHANDLE CXHANDLE, const TCHAR *mathExpression, const TCHAR *argvarList);


/**************************************************************************
       Evaluate prepared math expression.
        The function evaluates, prepared by Build() math expression for a given values.
        Arguments:
			equHandle - handle to context.
            arguments  array of values for a varList in Build().
        Return:
               Result of evaluation.
***************************************************************************/
TCalc __stdcall CalcFor(CXHANDLE CXHANDLE, const TCalc *arguments);


/**************************************************************************
     Parse math expression.
        The function prepares math expression but doesnt evaluate it. 
        To evaluate one for a given values, Run() should be called. 
        Functions Compile() and Run() should be used when the same math 
        expression must be evaluated for a large set of points. If number of 
        points is small (not more than 3)  function Evaluate() may work faster.
        Arguments:
			equHandle - handle to context.
            mathExpression  math expression as a string to be parsed.
            varList  coma separated list of named variables. For example x,y,z.
            arguments  array of values for a varList.
        Return:
               True if the call succeeded.
***************************************************************************/
bool __stdcall Compile(CXHANDLE CXHANDLE, const TCHAR *mathExpression, const TCHAR *argvarList, const TCalc *arguments);


/**************************************************************************
       Evaluate prepared math expression.
        The function evaluates, prepared by Compile() math expression for a given values.
        This function has no arguments. Third argument in Compile function represents an array 
        of TCalc values that user can change before calling Run function.
		Arguments:
			equHandle - handle to context.
        Return:
               Result of evaluation.
***************************************************************************/
TCalc __stdcall Run(CXHANDLE CXHANDLE);

/**************************************************************************
       Add a user defined function to EquTranslator.
       Arguments:
			equHandle - handle to context.
            funcName - name of the function as a string. This name must be 
					used in math expression.
            pFunction - pointer to user defined function.
        Return:
               True if the call has succeeded.
***************************************************************************/
bool __stdcall AddFunction(CXHANDLE CXHANDLE, const TCHAR* funcName, MATHFUNCTION_PTR pFunction);


/**************************************************************************
    Remove user defined function from EquTranslator.
        Arguments:
			equHandle - handle to context.
            funcName - name of the function as a string. 
        Return:
               True if the call has succeeded.
***************************************************************************/
bool __stdcall RemoveFunction(CXHANDLE CXHANDLE, const TCHAR* funcName);




#if defined(__cplusplus)
 }
#endif
 