// CxTranslator.cpp : Defines the entry point for the console application.
// 

#include <iostream>
#include <complex>
#include "CxWin32.h"

#ifdef UNICODE
#pragma comment(lib, "CxWin32W.lib")
#else
#pragma comment(lib, "CxWin32A.lib")
#endif


// Error handler
void  ErrorHandler(int errCode, const TCHAR* message, const TCHAR* message2)
{
	std::cout<<"Error: "<<message<<": "<<message2<<std::endl;
	throw errCode;
}


//User defined function
TCalc  pow3(const TCalc &x)
{
	//Must convert to std::Complex first
	std::complex<double> cmplx(x.real, x.imag);  

	//Do calculation
	std::complex<double> tmp =  cmplx*cmplx*cmplx;

	//Transform result to TCalc data type
	TCalc result = {tmp.real(), tmp.imag()};
	return result;
}


//	Operator overload example ... just for this sample
std::complex<double>& operator += (std::complex<double>& cmp, TCalc& cx)
{
	cmp.real(cmp.real() + cx.real);
	cmp.imag(cmp.imag() + cx.imag);
	return cmp;
}

TCalc& operator += (TCalc& cmp, TCalc& cx)
{
	cmp.real = cmp.real + cx.real;
	cmp.imag = cmp.imag + cx.imag;
	return cmp;
}


int _tmain(int argc, _TCHAR* argv[])
{
	TCHAR *mathExp = _T("x+2.3i*(1-sin(y/3.14)^2)-z");
	TCHAR *varList = _T("x,y,z");
	TCalc args[] = {{1.1, 2.5}, {1.7e-2, 0.32},	{3.14, 2.92}};
	TCalc res = {0,0};
	std::complex<double> cmpResult(0, 0);
	CXHANDLE cxHnd = NULL;


	//Initialize CxTranslator algorithm before we can use any function.
	cxHnd =  InitCxTranslator(
		ErrorHandler   //Error callback function
		);

	try
	{
		//Evaluate math expression for a given point "args"
		res = Evaluate(cxHnd, mathExp, varList, args); 

		args[0].real = 2.3; args[0].imag = 1.3; 
		args[1].real = -5;	args[1].imag = 0.123;
		res = Evaluate(cxHnd, _T("x/y+5-3.1j+(x^2-y^2)"), _T("x,y"), args); 

		//Add user's function to CxTranslator
		AddFunction(cxHnd, _T("p3"), pow3); 

		//Calculate a set of points for one math expression
		//Build it first
		Build(cxHnd, _T("3.14*p3(x)+sin(y)-sqrt(z+x)"), _T("x,y,z"));

		
		for(int i=0; i< 100; i++)
		{
			args[0].real = args[0].real + 1; args[0].imag = i; 
			args[1].real = i/200;	args[1].imag = 0.005*i;
			args[2].real = -5;	args[2].imag = args[0].real * 0.123;
						
			//Do calculation for a given point
			cmpResult += CalcFor(cxHnd, args);
		}
		
		//Another functions to calculate a set of points for a given expression
		Compile(cxHnd, mathExp, varList, args); 

		TCalc cxResult = {0, 0};
		for(int i=0; i< 10000; i++)
		{
			args[0].real = args[0].real + 0.0001; args[0].imag = i; 
			args[1].real = i/2000;	args[1].imag = i/10000;
			args[2].real = -5;	args[2].imag = args[0].real * 0.0123;
		
			//Do calculation for a given point
			cxResult += Run(cxHnd);
		}


		//Negative test
		RemoveFunction(cxHnd, _T("p3"));
		res = Evaluate(cxHnd, _T("3.14*p3(x)+sin(y)-sqrt(z+x)"), _T("x,y,z"), args);


	}
	catch(int errCode)
	{
		std::cout<<"Error code: "<<errCode<<std::endl;
	}
		

	//Release resources
	CloseCxTranslator(cxHnd); 

	return 0;
}


